from typing import Literal, Any, Optional, List, Mapping
from theia import Theia
from pulumi import ResourceOptions
from theia.aws.network.vpc import Vpc
from theia.aws.security.sg import SecurityGroup, SecurityGroupSettings


class dependentBaseTest:
    def __init__(self,
                 project: str,
                 stack: str,
                 parent_stack: str,
                 region: str,
                 backend_url: str,
                 sgSettings: Optional[List[SecurityGroupSettings]] = [SecurityGroupSettings(cidr_blocks=["0.0.0.0/0"])],
                 description: Optional[str] = None,
                 tags: Optional[Mapping[str, str]] = None):
        
        self.project = project
        self.stack = stack
        self.parent_stack = parent_stack
        self.region = region
        self.backend_url = backend_url
        self.sgSettings = sgSettings
        self.description = description
        self.tags = tags

    def infra(self):

        def program():

            targetVpc = Vpc.import_(stack=f"organization/{self.project}/{self.parent_stack}" , 
                                    resource_name=f"{self.project}-{self.parent_stack}-vpc")
            sg = SecurityGroup(resource_name=f"{self.project}-{self.stack}-sg",
                               vpc_id=targetVpc.id,
                               settings=self.settings,
                               opts=ResourceOptions(depends_on=targetVpc))
            
        return Theia(project=self.project, stack=self.stack, program=program, region=self.region, backend_url=self.backend_url, stack_imports=[f"organization/{self.project}/{self.parent_stack}"])

    def __call__(self, command: Literal['preview', 'up', 'destroy', 'refresh']) -> Any:
        t = self.infra()
        getattr(t, command)()

